package com.altimetrik.pmis.service.impl;

import com.altimetrik.pmis.core.entity.*;
import com.altimetrik.pmis.core.enums.Status;
import com.altimetrik.pmis.mapper.MetricsMapper;
import com.altimetrik.pmis.model.Metric;
import com.altimetrik.pmis.model.MetricIndicator;
import com.altimetrik.pmis.model.WeeklyMetricReport;
import com.altimetrik.pmis.repository.*;
import com.altimetrik.pmis.service.MetricsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;


@Service
public class MetricsServiceImpl implements MetricsService {
    private final AccountMetricsRepository accountMetricsRepository;
    private final MetricsMapper metricsMapper;

    private final AccountRepository accountRepository;

    private final IndicatorsRepository indicatorsRepository;

    private final ProjectRepository projectRepository;

    private final ProjectMetricRepository projectMetricRepository;

    private final WeeklyMetricRepository weeklyMetricRepository;

    private final TrendRepository trendRepository;

    @Autowired
    public MetricsServiceImpl(AccountMetricsRepository metricsRepository, AccountRepository accountRepository,
                              IndicatorsRepository indicatorsRepository, ProjectRepository projectRepository,
                              ProjectMetricRepository projectMetricRepository, WeeklyMetricRepository weeklyMetricRepository,
                              TrendRepository trendRepository) {
        this.accountMetricsRepository = metricsRepository;
        this.accountRepository = accountRepository;
        this.indicatorsRepository = indicatorsRepository;
        this.projectRepository = projectRepository;
        this.projectMetricRepository = projectMetricRepository;
        this.weeklyMetricRepository = weeklyMetricRepository;
        this.trendRepository = trendRepository;
        this.metricsMapper = MetricsMapper.INSTANCE;
    }


    @Override
    public Metric saveAccountMetric(Metric metric) {
        if (Objects.isNull(metric) || CollectionUtils.isEmpty(metric.getMetricIndicators())) {
            return null;
        }
        Optional<AccountEntity> accountEntityOptional = accountRepository.findById(metric.getAccountId());
        Optional<List<IndicatorEntity>> indicatorEntitiesOptional = indicatorsRepository.findByIdIn(metric.getMetricIndicators().stream().map(MetricIndicator::getIndicatorId).toList());
        return accountEntityOptional.isPresent()
                ? metricsMapper.accountMetricEntitiesToDomain(accountMetricsRepository.saveAllAndFlush(metricsMapper.domainToEntity(metric, accountEntityOptional.get(), indicatorEntitiesOptional)))
                : null;
    }

    @Override
    public Metric getAccountMetricsByAccountId(Long accountId) {
        List<AccountMetricEntity> accountMetricEntities = accountMetricsRepository.findByAccountIdAndIsActive(accountId, true);

        return metricsMapper.accountMetricEntitiesToDomain(accountMetricEntities);
    }

    @Override
    public Metric getProjectMetricsByProjectId(Long projectId) {
        List<ProjectMetricEntity> projectMetricEntities = projectMetricRepository.findByProjectIdAndIsActive(projectId, true);

        return metricsMapper.projectMetricEntitiesToDomain(projectMetricEntities);
    }

    @Override
    public Metric saveProjectMetric(Metric metric) {
        if (Objects.isNull(metric) || CollectionUtils.isEmpty(metric.getMetricIndicators())) {
            return null;
        }
        Optional<ProjectEntity> projectEntityOptional = projectRepository.findById(metric.getProjectId());
        Optional<List<IndicatorEntity>> indicatorEntitiesOptional = indicatorsRepository.findByIdIn(metric.getMetricIndicators().stream().map(MetricIndicator::getIndicatorId).toList());
        return projectEntityOptional.isPresent()
                ? metricsMapper.projectMetricEntitiesToDomain(projectMetricRepository.saveAllAndFlush(metricsMapper.domainToEntity(metric, projectEntityOptional.get(), indicatorEntitiesOptional)))
                : null;

    }

    @Override
    @Transactional
    public WeeklyMetricReport getOrCreateWeeklyMetricReportForProject(Long accountId, Long projectId, LocalDate reportDate) {
        WeeklyMetricReport weeklyMetricReport = null;
        if (null == accountId || null == projectId || null == reportDate) {
            //TODO:throw exception
            return weeklyMetricReport;
        }
        List<WeeklyMetricEntity> weeklyMetricEntities = weeklyMetricRepository.findByProjectId(projectId);
        if (!CollectionUtils.isEmpty(weeklyMetricEntities)) {
            List<TrendEntity> trendEntities = getWeeklyTrend(projectId, reportDate);
            List<Status> weeklyTrend = !CollectionUtils.isEmpty(trendEntities) ? trendEntities.stream().map(TrendEntity::getOverallStatus).collect(Collectors.toList()) : null;
            weeklyMetricReport = metricsMapper.mapWeeklyMetricEntityToWeeklyMetricReport(weeklyMetricEntities, reportDate, weeklyTrend);
        } else {
            createWeeklyMetricForProjectId(accountId, projectId, reportDate);
            List<WeeklyMetricEntity> weeklyMetricEntitiesAfterSave = weeklyMetricRepository.findByProjectId(projectId);
            List<TrendEntity> trendEntities = getWeeklyTrend(projectId, reportDate);
            List<Status> weeklyTrend = !CollectionUtils.isEmpty(trendEntities) ? trendEntities.stream().map(TrendEntity::getOverallStatus).collect(Collectors.toList()) : null;
            weeklyMetricReport = metricsMapper.mapWeeklyMetricEntityToWeeklyMetricReport(weeklyMetricEntitiesAfterSave, reportDate, weeklyTrend);
        }
        return weeklyMetricReport;

    }

    @Override
    @Transactional
    public WeeklyMetricReport updateWeeklyReportForProject(WeeklyMetricReport weeklyMetricReport) {
        WeeklyMetricReport weeklyMetricReportResponse = null;
        TrendEntity trendEntity = TrendEntity.builder().id(weeklyMetricReport.getTrendId()).overallStatus(weeklyMetricReport.getCurrentWeekOverallStatus()).build();

        List<WeeklyMetricEntity> weeklyMetricEntities = metricsMapper.mapWeeklyMetricReportToWeeklyMetricEntity(weeklyMetricReport, trendRepository.save(trendEntity));
        if (!CollectionUtils.isEmpty(weeklyMetricEntities)) {
            weeklyMetricRepository.saveAllAndFlush(weeklyMetricEntities);
            List<WeeklyMetricEntity> weeklyMetricEntitiesAfterSave = weeklyMetricRepository.findByProjectId(weeklyMetricReport.getProjectId());
            List<TrendEntity> trendEntities = getWeeklyTrend(weeklyMetricReport.getProjectId(), weeklyMetricReport.getReportDate());
            List<Status> weeklyTrend = !CollectionUtils.isEmpty(trendEntities) ? trendEntities.stream().map(TrendEntity::getOverallStatus).collect(Collectors.toList()) : null;
            weeklyMetricReportResponse = metricsMapper.mapWeeklyMetricEntityToWeeklyMetricReport(weeklyMetricEntitiesAfterSave, weeklyMetricReport.getReportDate(), weeklyTrend);

        }
        return weeklyMetricReportResponse;
    }

    private void createWeeklyMetricForProjectId(Long accountId, Long projectId, LocalDate reportDate) {
        List<AccountMetricEntity> accountMetricEntities = accountMetricsRepository.findByAccountIdAndIsActive(accountId, true);
        List<ProjectMetricEntity> projectMetricEntities = projectMetricRepository.findByProjectIdAndIsActive(projectId, true);
        TrendEntity trendEntity = TrendEntity.builder().project(ProjectEntity.builder().id(projectId).build()).reportDate(reportDate).build();
        List<WeeklyMetricEntity> weeklyMetricEntities = metricsMapper.mapWeeklyMetricEntity(accountMetricEntities, projectMetricEntities, trendRepository.save(trendEntity));
        if (!CollectionUtils.isEmpty(weeklyMetricEntities)) {
            weeklyMetricRepository.saveAllAndFlush(weeklyMetricEntities);
        }

    }

    private List<TrendEntity> getWeeklyTrend(Long projectId, LocalDate reportDate) {
        return trendRepository.findTop4ByProjectIdAndReportDateOrderByReportDateDesc(projectId, reportDate);
    }
}
