package com.altimetrik.pmis.service.impl;

import com.altimetrik.pmis.core.entity.ProjectEntity;
import com.altimetrik.pmis.core.entity.ProjectSummaryEntity;
import com.altimetrik.pmis.mapper.ProjectMapper;
import com.altimetrik.pmis.mapper.ProjectSummaryMapper;
import com.altimetrik.pmis.model.CreateProjectSummaryRequest;
import com.altimetrik.pmis.model.Project;
import com.altimetrik.pmis.model.ProjectSummary;
import com.altimetrik.pmis.model.ProjectSummaryResponse;
import com.altimetrik.pmis.repository.ProjectRepository;
import com.altimetrik.pmis.repository.ProjectSummaryRepository;
import com.altimetrik.pmis.service.AccountService;
import com.altimetrik.pmis.service.ProjectService;
import io.micrometer.common.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class ProjectServiceImpl implements ProjectService {
    private final ProjectRepository projectRepository;
    private final AccountService accountService;
    private final ProjectMapper projectMapper;

    private final ProjectSummaryRepository projectSummaryRepository;

    private final ProjectSummaryMapper projectSummaryMapper;

    @Autowired
    public ProjectServiceImpl(ProjectRepository projectRepository, AccountService accountService, ProjectSummaryRepository projectSummaryRepository) {
        this.projectRepository = projectRepository;
        this.accountService = accountService;
        this.projectSummaryRepository = projectSummaryRepository;
        this.projectSummaryMapper = ProjectSummaryMapper.INSTANCE;
        this.projectMapper = ProjectMapper.INSTANCE;
    }

    @Override
    public Project save(Long accountId, Project project) {
        project.setAccount(accountService.get(accountId));
        return projectMapper.entityToDomain(projectRepository.saveAndFlush(projectMapper.domainToEntity(project)));
    }

    @Override
    public Project get(Long id) {
        Optional<ProjectEntity> projectEntity = projectRepository.findById(id);
        if (projectEntity.isEmpty()) {
            // TODO: throw exception
        }
        return projectMapper.entityToDomain(projectEntity.get());
    }

    @Override
    public List<Project> getByAccount(Long accountId) {
        return projectRepository.findByAccountId(accountId).stream().map(projectEntity -> projectMapper.entityToDomain(projectEntity)).toList();
    }

    @Override
    public ProjectSummaryResponse createProjectSummary(CreateProjectSummaryRequest createProjectSummaryRequest) {
        List<ProjectSummary> projectSummaries = null;

        Optional<List<ProjectSummaryEntity>> projectSummaryEntitiesOptional = StringUtils.isEmpty(createProjectSummaryRequest.getAccountName())
                ? projectSummaryRepository.findByProjectAccountIdAndReportDate(createProjectSummaryRequest.getAccountId(), createProjectSummaryRequest.getReportDate())
                : projectSummaryRepository.findByProjectAccountIdAndProjectAccountNameAndReportDate(createProjectSummaryRequest.getAccountId(), createProjectSummaryRequest.getAccountName(), createProjectSummaryRequest.getReportDate());

        if (projectSummaryEntitiesOptional.isPresent() && !CollectionUtils.isEmpty(projectSummaryEntitiesOptional.get())) {
            projectSummaries = projectSummaryMapper.projectSummaryEntitiesToDomains(projectSummaryEntitiesOptional.get());
            createProjectSummaryRequest.setAccountName(StringUtils.isEmpty(createProjectSummaryRequest.getAccountName())
                    ? projectSummaryEntitiesOptional.get().get(0).getProject().getAccount().getName()
                    : createProjectSummaryRequest.getAccountName());
            return ProjectSummaryResponse.builder()
                    .accountId(createProjectSummaryRequest.getAccountId())
                    .accountName(createProjectSummaryRequest.getAccountName())
                    .reportDate(createProjectSummaryRequest.getReportDate())
                    .projectSummaries(projectSummaries)
                    .build();
        }
        List<ProjectEntity> projectEntities = projectRepository.findByAccountId(createProjectSummaryRequest.getAccountId());

        List<ProjectSummaryEntity> projectSummaryEntities = projectEntities.stream().map(projectEntity -> {
            return ProjectSummaryEntity.builder().project(projectEntity).reportDate(createProjectSummaryRequest.getReportDate()).build();
        }).collect(Collectors.toList());


        projectSummaries = projectSummaryMapper.projectSummaryEntitiesToDomains(projectSummaryRepository.saveAllAndFlush(projectSummaryEntities));
        createProjectSummaryRequest.setAccountName(StringUtils.isEmpty(createProjectSummaryRequest.getAccountName())
                ? projectSummaryEntities.get(0).getProject().getAccount().getName()
                : createProjectSummaryRequest.getAccountName());
        return ProjectSummaryResponse.builder()
                .accountId(createProjectSummaryRequest.getAccountId())
                .accountName(createProjectSummaryRequest.getAccountName())
                .reportDate(createProjectSummaryRequest.getReportDate())
                .projectSummaries(projectSummaries)
                .build();
    }

    @Override
    public ProjectSummaryResponse getProjectSummaryByAccountAndReportDate(Long accountId, String accountName, LocalDate reportDate) {
        Optional<List<ProjectSummaryEntity>> projectSummaryEntitiesOptional = StringUtils.isEmpty(accountName)
                ? projectSummaryRepository.findByProjectAccountIdAndReportDate(accountId, reportDate)
                : projectSummaryRepository.findByProjectAccountIdAndProjectAccountNameAndReportDate(accountId, accountName, reportDate);
        if (projectSummaryEntitiesOptional.isPresent()) {
            accountName = StringUtils.isEmpty(accountName) ? projectSummaryEntitiesOptional.get().get(0).getProject().getAccount().getName() : accountName;
            List<ProjectSummary> projectSummaries = projectSummaryMapper.projectSummaryEntitiesToDomains(projectSummaryEntitiesOptional.get());
            return ProjectSummaryResponse.builder().accountId(accountId).accountName(accountName).projectSummaries(projectSummaries).build();
        } else {
            return createProjectSummary(CreateProjectSummaryRequest.builder().accountId(accountId).reportDate(reportDate).build());
        }
    }

    @Override
    public ProjectSummary updateProjectSummary(ProjectSummary projectSummary) {
        return projectSummaryMapper.projectSummaryEntityToDomain(projectSummaryRepository.saveAndFlush(projectSummaryMapper.mapProjectSummaryDomainToEntity(projectSummary)));
    }

    @Override
    public List<ProjectSummary> updateMultipleProjectSummary(List<ProjectSummary> projectSummaries) {
        return projectSummaryMapper.projectSummaryEntitiesToDomains(projectSummaryRepository.saveAllAndFlush(projectSummaryMapper.projectSummaryDomainsToEntities(projectSummaries)));
    }
}
